% 3-RRPaR Spatial Parallel Robot
clc
clear
close all
global r a b h mp mb ma g dinv_J
%% parameters
r = 0.16; %mm
a = 0.2; %mm
b = 0.2; %mm
h = 0.12; %mm
mp = 0.4; %kg
ma = 0.3; %kg
mb = 0.1; %kg
g = 9.8; %m/s^2
%% Initilization
tow = [0 0 0];
p = [0 0 0];
q1_int = [0.95,0.421,0.95];dq1 = [1.426,0,-1.426];
q3_int = [1 1.5 2];q2_int = [1.7 1.8 1.7];
int_e = [0 0 0];e = [0.006 0 -0.006];de = [0.0002 0 -0.0002];
dinv_J = [-9.44e-02  -1.63e-01  -9.44e-02;5.5e-2  0  -5.5e-2;-1.76e-01  0  1.76e-01];
IC = [tow,p,q1_int,dq1,int_e,e,0 0 0,q3_int,q2_int,zeros(1,21)].';
Tspan=linspace(0,4,1e5); %time span
%% ODE solving
[T,Q] = ode45(@myfun,Tspan,IC);
%% Inverse kinematics-Joint variables
px = -0.1*sin(pi*T);
py = 0.1*cos(pi*T); 
pz = 0.29-0.05*cos((pi*T)/2);
   
for i=1:3
    phi(i) = (pi/2)*(i-1);

            cx(:,i) = cos(phi(i)).*px+sin(phi(i)).*py+h*ones(numel(T),1)-r*ones(numel(T),1);
            cy(:,i) = -sin(phi(i)).*px+cos(phi(i)).*py;
            cz(:,i) = pz;
            
            q3(:,i) = acos(cy(:,i)./b);
         
            k = (cx(:,i).^2+cy(:,i).^2+cz(:,i).^2-(a^2+b^2))./(2*a*b*sin(q3(:,i)));
            q2(:,i) = acos(k);
           
            g1(:,i) = a+b*cos(q2(:,i)).*sin(q3(:,i));
            g2(:,i) = b*sin(q2(:,i)).*sin(q3(:,i));
            q1(:,i) = atan2((-g2(:,i).*cx(:,i)+g1(:,i).*cz(:,i)),(g1(:,i).*cx(:,i)+g2(:,i).*cz(:,i)));

end
%% error (IAE)
e_joint1 = abs(trapz(T,(q1(:,1)-Q(:,7))))
e_joint2 = abs(trapz(T,(q1(:,2)-Q(:,8))))
e_joint3 = abs(trapz(T,(q1(:,3)-Q(:,9))))
IAE = e_joint1+e_joint2+e_joint3
%% position
q11=diff(Q(:,19))./diff(T);
q12=diff(Q(:,20))./diff(T);
q13=diff(Q(:,21))./diff(T);
tt=0:(T(end)/(length(q11)-1)):T(end);

figure
plot(T,radtodeg(Q(:,7)),'b')
hold on
plot(T,radtodeg(Q(:,8)),'g')
plot(T,radtodeg(Q(:,9)),'r')

plot(tt,radtodeg(q11),'--')
plot(tt,radtodeg(q12),'--')
plot(tt,radtodeg(q13),'--')


legend('joint1','joint2','joint3'),grid on
xlabel('Time(s)'),ylabel('Joint Angles(degree)')

%% velocities
dq11=diff(Q(:,31))./diff(T);
dq12=diff(Q(:,32))./diff(T);
dq13=diff(Q(:,33))./diff(T);
tt=0:(T(end)/(length(dq11)-1)):T(end);

figure
plot(T,Q(:,10),'b')
hold on
plot(T,Q(:,11),'g')
plot(T,Q(:,12),'r')

plot(tt,dq11,'--')
plot(tt,dq12,'--')
plot(tt,dq13,'--')

legend('joint1','joint2','joint3'),grid on
xlabel('Time(s)'),ylabel('velocities(rad/s)')
%% Acceleration
d2q11=diff(Q(:,10))./diff(T);
d2q12=diff(Q(:,11))./diff(T);
d2q13=diff(Q(:,12))./diff(T);
tt=0:(T(end)/(length(d2q11)-1)):T(end);
% desired
q11_2dot=diff(Q(:,34))./diff(T);
q12_2dot=diff(Q(:,35))./diff(T);
q13_2dot=diff(Q(:,36))./diff(T);

figure
plot(tt,d2q11,'b')
hold on
plot(tt,d2q12,'g')
plot(tt,d2q13,'r')

plot(tt,q11_2dot,'--')
plot(tt,q12_2dot,'--')
plot(tt,q13_2dot,'--')

legend('joint1','joint2','joint3'),grid on
xlabel('Time(s)'),ylabel('Accelerationes(rad/s^2)')
%% torque inputs 
tow1=diff(Q(:,1))./diff(T);
tow2=diff(Q(:,2))./diff(T);
tow3=diff(Q(:,3))./diff(T);
tt=0:(T(end)/(length(tow1)-1)):T(end);
%torque1 plot
figure
plot(tt,tow1,'b')

%torque2 plot
hold on
plot(tt,tow2,'g')

%torque3 plot
hold on
plot(tt,tow3,'r')
grid on 

legend('joint1','joint2','joint3'),grid on
xlabel('Time(s)'),ylabel('Joint Torques(N.m)')
%% position of mobile platform
px = -0.1*sin(pi*T);
py = 0.1*cos(pi*T); 
pz = 0.29-0.05*cos((pi*T)/2);

x = diff(Q(:,4))./diff(T);
y = diff(Q(:,5))./diff(T);
z = diff(Q(:,6))./diff(T);
tt=0:(T(end)/(length(x)-1)):T(end);

figure
plot(tt,x,'b')
hold on
plot(tt,y,'g')
plot(tt,z,'r')

plot(T,px,'--')
plot(T,py,'--')
plot(T,pz,'--')

legend('x','y','z'),grid on
xlabel('Time(s)'),ylabel('end-effector position(m)')
%% gain
kp1 = diff(Q(:,40))./diff(T);
kp2 = diff(Q(:,41))./diff(T);
kp3 = diff(Q(:,42))./diff(T);

kd1 = diff(Q(:,43))./diff(T);
kd2 = diff(Q(:,44))./diff(T);
kd3 = diff(Q(:,45))./diff(T);

kI1 = diff(Q(:,46))./diff(T);
kI2 = diff(Q(:,47))./diff(T);
kI3 = diff(Q(:,48))./diff(T);
tt=0:(T(end)/(length(kp1)-1)):T(end);

figure
plot(tt,kp1)
hold on
plot(tt,kp2)
plot(tt,kp3)
legend('kp1','kp2','kp3'),grid on
xlabel('Time(s)'),ylabel('gain')

figure
plot(tt,kd1)
hold on
plot(tt,kd2)
plot(tt,kd3)
legend('kd1','kd2','kd3'),grid on
xlabel('Time(s)'),ylabel('gain')

figure
plot(tt,kI1)
hold on
plot(tt,kI2)
plot(tt,kI3)
legend('kI1','kI2','kI3'),grid on
xlabel('Time(s)'),ylabel('gain')